// (C) 2003 Eli Curtz <eli@nuprometheus.com>
// Licensed under the terms of the GNU Public License version 2

#include "f2a.h"
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>

int f2a_multiboot(char* fileName)
{
	int	file;
	sendmsg	sm;
	size_t	fileSize;
	char	fileBuffer[LOADUNIT];
	size_t	bytesRead;
	struct stat st;
	int j;
   
	if (stat(fileName, &st) == -1)
	{
		perror(fileName);
		return -1;
	}
	fileSize = st.st_size;

	if ((file = open(fileName, O_RDONLY)) == -1)
		perror(fileName);

#if 0
	// Round up to 32k, might not be necessary for multiboot
	if(fileSize & (32768 - 1))
		fileSize += 32768;
	fileSize &= ~(32768 - 1);
#endif

	// Initialize command buffer
	memset(&sm, 0, sizeof(sm));
	sm.command = CMD_WRITEDATA;
	sm.size = fileSize;
	sm.magic = MAGIC_NUMBER;
	sm.unknown = 0x6; // write in ram(?) (was: no idea what this is...)
	sm.address = GBA_EWRAM;
	sm.sizekb = fileSize / 1024;

	// Send the data to EWRAM
	if (sendf2a(sm) == -1)
	{
		printf("F2A Error: Unable to initiate Multiboot download\n");
		return -1;
	}

#if 0
	// Another spot that seems to require some time
	sleep(3);
#endif

	for (j = 0; j < fileSize; j += LOADUNIT)
	{
		if (verbose > 0)
			printf("Writing Multiboot data %d\n", (int)j);
	   
		// Read in LOADUNIT of data from the file
		bytesRead = read(file, fileBuffer, LOADUNIT);
		if (bytesRead == -1)
		{
			perror(fileName);
			return -1;
		}
	   
		// Write LOADUNIT of data to the EWRAM
		if (usb_write(fileBuffer, bytesRead) == -1)
		{
			printf("F2A Error: Unable to write Multiboot data\n");
			return -1;
		}
	}
	close(file);
   
	// Launch the program
	memset(&sm, 0, sizeof(sm));
	sm.command = CMD_WRITEDATA;
	sm.magic = MAGIC_NUMBER;
	sm.unknown = 0x8; // boot in sram(?) (was: no idea what this is...)
	sm.address = GBA_EWRAM;
	   
	if (sendf2a(sm) == -1)
	{
		printf("F2A Error: Unable to initiate Multiboot download\n");
		return -1;
	}
   
	return 1;
} 
